package org.jconfig;

import java.util.*;
import java.lang.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import java.awt.*;
import java.awt.event.*;
import org.jconfig.items.*;
import org.jconfig.visual.*;

/**
 *  <p>package:    org.jconfig</p>
 *  <p>file:       ConfigApp.java</p>
 * $Id: ConfigApp.java 274 2004-04-19 08:12:45Z tgl $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 *	Configuration application to create configuration files
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class ConfigApp extends javax.swing.JFrame implements ActionListener {
    
    private JPopupMenu          ItemsPopUp = null;
    private TreeItem            topItem, topConf;
    private JTree               ConfTree;
    private DefaultTreeModel    ConfModel; 
    private ConfigPanel         AttrPanel;
    private JScrollPane             TreeView, AttrView;
    private DefaultMutableTreeNode  topnode, copynode;
    private JFileChooser            fc;
    private boolean             cutandpaste;

    /** Creates a new instance of ConfigApp */
    public ConfigApp() {
        ParseConfig ();
        BuildItemsPopUp ();
        BuildConfTree ();
        InitComponents ();
    }
    
    public void actionPerformed(ActionEvent e) {
        
        ConfigMenuItem source = (ConfigMenuItem)(e.getSource());

        DefaultMutableTreeNode node = (DefaultMutableTreeNode) ConfTree.getLastSelectedPathComponent();
        if (node == null) 
            return;
        
        // Functions
        if (source.item == null) {
            switch (source.menuid) {
                // Delete everything except root node
                case 0: {
                    if (!node.isRoot()) {
                        ((TreeItem) node.getUserObject()).RemoveFromParent();
                        ConfModel.removeNodeFromParent(node);
                    }
                    return;
                }
                // Cut
                case 1:
                    cutandpaste = true;
                    if (!node.isRoot())
                        copynode = node;
                    else
                        copynode = null;
                    return;
                // Copy
                case 2:
                    cutandpaste = false;
                    if (!node.isRoot())
                        copynode = node;
                    else
                        copynode = null;
                    return;
                // Paste
                case 3:
                    if (copynode == null)
                        break;
                    TreeItem it = (TreeItem) copynode.getUserObject();
                    TreeItem newit = ((TreeItem)node.getUserObject()).InsertItem(new TreeItem (it, -1), false);
                    if (newit != null) {
                        DefaultMutableTreeNode child = new DefaultMutableTreeNode (newit);
                        ConfModel.insertNodeInto(child, node, node.getChildCount());
                        BuildNodes (child);
                        if (cutandpaste == true) {
                            ((TreeItem) copynode.getUserObject()).RemoveFromParent();
                            ConfModel.removeNodeFromParent(copynode);
                        }
                    }
                    cutandpaste = false;
                    return;
                // Move up
                case 4:
                    return;
                // Move Down
                case 5:
                    return;
                // Save
                case 6:
                    SaveTree (topnode);
                    return;
                case 7:
                    LoadTree ();
                    return;
                // Not here    
                default:
                    return;
            }
            return;
        }
        // Add external configured 
        TreeItem it = (TreeItem)node.getUserObject();
        TreeItem newit = it.InsertItem (source.item);
        if (newit == null)
            return;
        DefaultMutableTreeNode child = new DefaultMutableTreeNode (newit);
        ConfModel.insertNodeInto(child, node, node.getChildCount());
        //ConfModel.reload(node);
        //ConfTree.scrollPathToVisible(new TreePath(child.getPath()));
        ConfTree.setSelectionPath(new TreePath(child.getPath()));
    }        
    
    // Save the tree recursive
    private void SaveTree (DefaultMutableTreeNode top) {
        ItemToXML encoder = ((TreeItem)top.getUserObject()).printFollow(null);
        if (encoder == null)
            return;

        if (fc.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
            return;
        
        try {
            FileWriter out = new FileWriter(fc.getSelectedFile().getAbsoluteFile());
            out.write (encoder.GetString());
            out.write ('\n');
            out.close();
        } catch (Exception e) {}
        SaveItemOffset();
    }

    private void SaveItemOffset () {
        String cfg = fc.getSelectedFile().getAbsolutePath();
        int last = cfg.lastIndexOf('.');
        cfg = cfg.substring(0,last) + ".conf";
        ArrayList fn = new ArrayList();

        try {
        String line;
        BufferedReader in = new BufferedReader(new FileReader(cfg));
        while ((line = in.readLine()) != null) {
            if (line.charAt(0) == '#')
                continue;
            String[] tags = line.split("=");
            if (tags == null || tags.length != 2)
                continue;
            fn.add (tags[1]);
        }
        in.close();
        } catch (Exception e) {return;}

        String files[] = new String[fn.size()];
        for (int i = 0; i < files.length; i++)
            files[i] = "";
        
        SaveItemOffsets ((TreeItem)topnode.getUserObject(), files);
        
        for (int i = 0; i < files.length; i++) {
            try {
                String p = fc.getSelectedFile().getAbsolutePath();
                int d = p.lastIndexOf(fc.getSelectedFile().separatorChar) + 1;
                String f = p.substring(0,d) + ((String)fn.get(i)) + ".offset";
                FileWriter out = new FileWriter(f);
                out.write (files[i]);
                out.write ('\n');
                out.close();
            } catch (Exception e) {}
        }
    }
    
    private void SaveItemOffsets (TreeItem item, String files[]) {
        int type = -1;
        String symbol = item.GetAttrValue("symbol");
        String value = item.GetAttrValue("offset");
        if (symbol != null && value != null && value.length() > 0) {
            symbol += "=";
            if (value != null) {
                if (value.indexOf(' ') == -1 && value.indexOf('=') == -1) 
                    symbol += value;
                else 
                    symbol += "\"" + value + "\"";
            }
            try {
                type = Integer.parseInt(item.GetAttrValue("targettype"));
                if (type >= 0 && type < files.length)
                    files[type] += symbol + "\n";
            } catch (NumberFormatException e) { }
        }
        for (int i = 0; i < item.items.size(); i++)
            SaveItemOffsets((TreeItem)item.GetItem(i), files);
    }    
    
    // Load the tree
    private void LoadTree () {
        
        int i;
        
        if (fc.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
            return;

        // parse the given tree items XML
        TreeItem item = new TreeItem("Read", null);
        TreeItemHandler Handler = new TreeItemHandler(item, false);
        try { 
            SAXParser p = (SAXParserFactory.newInstance()).newSAXParser();
            p.parse(fc.getSelectedFile().getAbsoluteFile(), Handler); 
        } 
        catch (Exception e) {e.printStackTrace();}
        
        TreeItem top = null;
        // Find the top node
        for (i = 0; i < item.items.size(); i++) {
            top = (TreeItem) item.items.get(i);
            if (top.GetName().compareTo("topitem") == 0)
                break;
            top = null;
        }
        if (top == null) {
            item.RemoveAll();
            return;
        }
        top.RemoveFromParent();
        
        // Remove all current nodes and items
        topConf.RemoveAll();
        
        SetItemConfig(top);
        
        topConf = top;
        topnode.removeAllChildren();
        topnode.setUserObject(topConf);
        ConfModel.reload(topnode);
        
        BuildNodes(topnode);
        ConfModel.reload(topnode);
    }

    private void BuildNodes (DefaultMutableTreeNode parent) {
        TreeItem item = (TreeItem) parent.getUserObject();
        for (int i = 0; i < item.items.size(); i++) {
            TreeItem subitem = (TreeItem) item.items.get(i);
            DefaultMutableTreeNode child = new DefaultMutableTreeNode (subitem);
            ConfModel.insertNodeInto(child, parent, parent.getChildCount());
            BuildNodes (child);
        }
    }
    
    private void SetItemConfig(TreeItem item) {
        ItemAttr[] attr = new ItemAttr[] {new ItemAttr("isconfig", "true"), item.GetAttr("object")};
        item.SetConfig(topItem.GetItemFollow(item.GetName(), attr));
        for (int i=0; i < item.items.size(); i++)
            SetItemConfig((TreeItem)item.items.get(i));
    }

    /** For now build an empty one */
    private void BuildConfTree () {
        topConf = new TreeItem ("Topconf", null, topItem.GetItemFollow("topitem"));
        topnode = new DefaultMutableTreeNode (topConf);
    }
    
    private void TreeMouseClicked (java.awt.event.MouseEvent evt) {
        switch (evt.getButton()) {
            case java.awt.event.MouseEvent.BUTTON2:
            case java.awt.event.MouseEvent.BUTTON3:    
                break;
            default: return;
        }
        TreePath p = ConfTree.getPathForLocation(evt.getX(),  evt.getY());
        if (p == null)
            return;
        ConfTree.setSelectionPath(p);
        // TreeItem = ConfTree.
        if (ItemsPopUp != null)
            ItemsPopUp.show(this, ConfTree.getWidth() - 50, evt.getY());
    }
    
    private void NodeSelect (TreeSelectionEvent e) {

        DefaultMutableTreeNode node = (DefaultMutableTreeNode) ConfTree.getLastSelectedPathComponent();
        if (node == null) 
            return;
        
        TreeItem it = (TreeItem)node.getUserObject();
        AttrPanel.SetConfigItems(it, false, -1);
    }
    
    private void BuildItemsPopUp (){
        
        TreeItem it = topItem.GetItem(0);
        
        if (it != null) {
            ItemsPopUp = new JPopupMenu("Config");
            JMenu add = new JMenu("Add");
        
            for (int i = 0; i < it.items.size(); i++) {
                String cfg = it.GetItem(i).GetAttrValue("isconfig");
                if (cfg == null || cfg.compareTo("true") != 0)
                    continue;
                ConfigMenuItem ci = new ConfigMenuItem(it.GetItem(i));
                ci.addActionListener(this);
                add.add(ci);
            }
            ItemsPopUp.add(add);
            ItemsPopUp.addSeparator();
            
            String[] fun = { "Delete", "Cut", "Copy", "Paste", "Move Up", "Move Down", "Save", "Load"};
            for (int i = 0; i < fun.length; i++) {
                ConfigMenuItem ci = new ConfigMenuItem(i, fun[i]);
                ci.addActionListener(this);
                ItemsPopUp.add(ci);
            }
        }    
    }
    
    private void ParseConfig () {
        // parse the given tree items XML
        topItem = new TreeItem("Top", null);
        TreeItemHandler Handler = new TreeItemHandler(topItem, false);
        try { 
            SAXParser p = (SAXParserFactory.newInstance()).newSAXParser();
            p.parse("./config.xml", Handler); 
        } 
        catch (Exception e) {e.printStackTrace();}
    }
    
    private void InitComponents () {
        setSize(new Dimension (600,400));

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        ConfModel = new DefaultTreeModel(topnode);
        ConfTree = new JTree(ConfModel);
        ConfTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        //Listen for when the selection changes.
        ConfTree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                NodeSelect (e);
            } 
        });    
        TreeView = new JScrollPane(ConfTree, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        
        AttrPanel = new ConfigPanel(false, BoxLayout.Y_AXIS);
        AttrView = new JScrollPane(AttrPanel,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        
        getContentPane().add(TreeView);
        getContentPane().add(AttrView);
        getContentPane().add(new JSplitPane (JSplitPane.HORIZONTAL_SPLIT, TreeView, AttrView));

        ConfTree.addMouseListener (new java.awt.event.MouseListener() {
            public void mouseClicked (java.awt.event.MouseEvent evt) {
                TreeMouseClicked (evt);
            }
            public void mousePressed (java.awt.event.MouseEvent evt) {
            }
            public void mouseReleased (java.awt.event.MouseEvent evt) {
            }
            public void mouseEntered (java.awt.event.MouseEvent evt) {
            }
            public void mouseExited (java.awt.event.MouseEvent evt) {
            }
        });
        fc = new JFileChooser();        
    }
    
    private void exitForm(java.awt.event.WindowEvent evt) {
        System.exit(0);
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        new ConfigApp().show();
    }
  
}
