package org.jconfig.visual;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.jconfig.items.*;
import org.jvisu.util.JVCronSelect;
import java.beans.*;

/**
 *  <p>package:    org.jconfig.visual</p>
 *  <p>file:       ConfigCronJobs.java</p>
 * $Id: ConfigCronJobs.java 274 2004-04-19 08:12:45Z tgl $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 *	Cronjob widget for configuration editor
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class ConfigCronJobs extends JVCronSelect implements PropertyChangeListener {
    
    public String   id;
    public TreeItem item, cfgitem;
    
    static protected final String PERMANENT_FOCUS_OWNER = "permanentFocusOwner";
    
    /** Creates a new instance of ConfigTextArea */
    public ConfigCronJobs(TreeItem it, TreeItem cfg, String name, int ulevel) {
        super();
        item = it;
        id = name;
        cfgitem = cfg;
        String txt;
        KeyboardFocusManager kfm = KeyboardFocusManager.getCurrentKeyboardFocusManager();
        kfm.addPropertyChangeListener( PERMANENT_FOCUS_OWNER, this );
        
        if (cfgitem.CheckAttrValue("storecontent", "true"))
            txt = it.GetContent();
        else
            txt = it.GetOrBuildAttrValue(id);
        setSelections(txt);
        
        this.setEnabled(ConfigPanel.ConfigCheckUserlevel(it, ulevel));
    }
    
    /** Called form propertychange, to find out, if we have lost the focus */
    private void handleFocusChange(Component c) {
        Component co[] = this.getComponents();
        for (int i = 0; i < co.length; i++)
            if (co[i] == c)
                return;
        if (cfgitem.CheckAttrValue("storecontent", "true"))
            item.SetContent(getSelections());
        else
            item.ModifyAttr(id, getSelections());
    }
    
    /** Called, if the requested property changes */
    public void propertyChange(PropertyChangeEvent evt) {
        KeyboardFocusManager kfm = KeyboardFocusManager.getCurrentKeyboardFocusManager();
        if ( kfm == evt.getSource() && PERMANENT_FOCUS_OWNER.equals( evt.getPropertyName() ) )
            handleFocusChange( (Component) evt.getNewValue() );
    }
}
