package org.jvisu.base;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.event.*;
import org.jvisu.comm.var.*;

/** Visual component to display numerical (double) values
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 *
 * <p>
 * $Id: JVSlider.java 318 2007-08-21 11:23:19Z mol $
 * </p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 *  JVSlider is a read/write component to display and edit numerical values.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVSlider extends JVBar implements PropertyChangeListener, MouseListener, MouseMotionListener {
    
    /** The mouse state */
    private int                 mousestate;
    /** Draw the bar */
    private boolean             drawbar = true;
    /** Draw the track */
    private boolean             drawtrack = true;
    /** The slider color    */
    private Color               sliderColor = Color.LIGHT_GRAY;
    /** The track color    */
    private Color               trackColor = Color.GRAY;
    /** The rectangle, where the slider is actually */
    private Rectangle           sliderRect = new Rectangle();
    /** The mouse position, where the slider was clicked */
    private Point               clickPoint = new Point();
    /** The mouse position, where the slider was dragged */
    private Point               dragPoint = new Point();
    /* The point, where the slider was, when drag started */
    private Point               startPoint = new Point();
    
    /** Create a new instance of JVSlider
     */
    public JVSlider() {
	addMouseListener(this);
        addMouseMotionListener(this);
    }

    /** Paint this component
     *  @param g the graphics context
     */
    public void paintComponent (Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        
        /* On recalc, set the insets before calling super paint */
        if (recalc) {
            if (orientation == 0) {
                insetx = (getWidth() * 5) / 100;
                insety = DEFAULT_INS;
            } else {
                insetx = DEFAULT_INS;
                insety = (getHeight() * 5) / 100;
            }
        }
        /* Recalc the scale */
        calcScale(g2);
        /* Draw a border, if selected */
        drawBorder(g2);
        /* Paint the bar, if requested */
        if (drawbar)
            drawBar(g2, actval, null);
        /* Paint the scale */
        drawScale(g2);
        /* Now paint the slider part */
        drawSlider(g2);
    }

    /** Draw the slider
     *  @param g2 the graphics context
     */
    protected void drawSlider(Graphics2D g2) {
        /* Get the position of the actual value as a factor of the bar length 
         * and calculate the length of the bar */
        double ext = (double) ((orientation == 0) ? barRect.width : barRect.height);
        int len = (int) (ext * calcPosition(actval, 0));
        /** Horizontal */
        if (orientation == 0) {
            if (drawtrack) {
                g2.setColor(trackColor);
                int h = (barRect.height * 30) / 100;
                if (h > 10)
                    h = 10;
                int y = (barRect.height - h) / 2;
                g2.fill3DRect(barRect.x, barRect.y + y, barRect.width, h, false);
            }
            g2.setColor(sliderColor);
            int x = barRect.x + (invers ? (barRect.width - len) : len );
            int h = barRect.height;
            int w = barRect.width > 200 ? 40 : (barRect.width * 20 / 100);
            sliderRect.setLocation(x - w / 2, barRect.y);
            sliderRect.setSize(w, h);
            g2.fill3DRect(sliderRect.x, sliderRect.y, w, h, true);
            g2.fill3DRect(x - 2, barRect.y, 4, h, false);
        } else {
            if (drawtrack) {
                g2.setColor(trackColor);
                int w = (barRect.width * 30) / 100;
                if (w > 10)
                    w = 10;
                int x = (barRect.width - w) / 2;
                g2.fill3DRect(barRect.x + x, barRect.y, w, barRect.height, false);
            }
            g2.setColor(sliderColor);
            int y = barRect.y + (invers ? len : (barRect.height - len));
            int w = barRect.width;
            int h = barRect.height > 200 ? 40 : (barRect.height * 20 / 100);
            sliderRect.setLocation(barRect.x, y - h / 2);
            sliderRect.setSize(w, h);
            g2.fill3DRect(sliderRect.x, sliderRect.y, w, h, true);
            g2.fill3DRect(barRect.x, y - 2, w, 4, false);
        }
    }
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
        if (evt.getSource() == this) {
            if (key.equals("editEnabled") || key.equals("readonly")) {
                repaint();
            }
        }
    }
    
    /** Mouse button was clicked
     *  @param e mouse event
     */
    public void mouseClicked(java.awt.event.MouseEvent evt) {
    }

    /** Mouse button was pressed
     *  @param e mouse event
     */
    public void mousePressed(java.awt.event.MouseEvent evt) {
        mousestate = 0;
        if (!isEditEnabled() || isReadonly())
            return;
        if (evt.getButton() != MouseEvent.BUTTON1)
            return;

        /* Store the point, remove the component offset */
        clickPoint.setLocation(evt.getPoint());
        startPoint.x = sliderRect.x - barRect.x;
        startPoint.y = sliderRect.y - barRect.y;
        if (sliderRect.contains(clickPoint)) {
            mousestate = 1;
            setCursor(Cursor.getPredefinedCursor(
             (orientation == 0) ? Cursor.E_RESIZE_CURSOR : Cursor.N_RESIZE_CURSOR));
        }
    }

    /** Mouse button was released
     *  @param e mouse event
     */
    public void mouseReleased(java.awt.event.MouseEvent evt) {
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        mousestate = 0;
    }

    /** Mouse entered area.
     *  @param evt mouse event
     */
    public void mouseEntered(MouseEvent evt) {
    }
    
    /** Mouse left area.
     *  @param evt mouse event
     */
    public void mouseExited(MouseEvent evt) {
        
    }

    /** Mouse dragged 
     *  @param evt mouse event
     */
    public void mouseDragged(MouseEvent evt) {

        if (mousestate != 1)
            return;
        
        dragPoint.setLocation(evt.getPoint());
        
        int len;
        int pos;
        if (orientation == 0) {
            len = barRect.width;
            pos = dragPoint.x - clickPoint.x;
            pos += startPoint.x + sliderRect.width / 2;
        } else {
            len = barRect.height;
            pos = len - (dragPoint.y - clickPoint.y);
            pos -= startPoint.y + sliderRect.height / 2;
        }
        if (invers)
            pos = len - pos;
        /* Limit the motion */
        if (pos > len)
            pos = len;
        if (pos < 0)
            pos = 0;
        
        try {
            setValue(calcValAtPosition(pos, len));
        } catch (ArithmeticException e) {
            mousestate = 0;
        }        
    }
    
    /** Mouse was moved
     *  @param evt mouse event
     */
    public void mouseMoved(MouseEvent evt) {
        
        int cursor = Cursor.DEFAULT_CURSOR;
        
        clickPoint.setLocation(evt.getPoint());
        
        if (sliderRect.contains(clickPoint) && isEditEnabled() &&  !isReadonly())
            cursor = Cursor.HAND_CURSOR;
                
        if (cursor != getCursor().getType())
            setCursor(Cursor.getPredefinedCursor(cursor));
    }

    /** These properties are not affecting the background
     *  drawing. So we must not fire a change event and
     *  can just call repaint(). This does not hurt
     *  unless somebody calls the setter methods 
     *  repeadeately. He will notice :)
     */
    
    /** Get the color of the slider knob
     * @return color of the slider knob
     */
    public Color getSliderColor() {
        return sliderColor;
    }
    
    /** Set the color of the slider knob
     * @param color the color of the slider knob
     *
     */
    public void setSliderColor(Color color) {
        sliderColor = color;
        repaint();
    }
    
    /** Get the color of the slider track
     * @return the color of the slider track
     */
    public Color getTrackColor() {
        return trackColor;
    }
    
    /** Set the color of the slider track
     * @param color the color of the slider track
     */
    public void setTrackColor(Color color) {
        trackColor = color;
        repaint();
    }
    
    /** Get wether the bar should be drawn with
     *  the given fill and alarm colors. 
     * @return true, if bar drawing is enabled
     */
    public boolean isDrawBarEnabled() {
        return drawbar;
    }
    
    /** Set if the bar should be drawn with
     *  the given fill and alarm colors. 
     *  @param ena true to enable bar drawing
     */
    public void setDrawBarEnabled(boolean ena) {
        drawbar = ena;
        repaint();
    }
    
    /** Get wether the slider track should be drawn with
     *  the given track color. 
     * @return true, if track drawing is enabled
     */
    public boolean isDrawTrackEnabled() {
        return drawtrack;
    }
    
    /** Set whether the slider track should be drawn with
     *  the given track color. 
     *  @param ena true to enable track drawing
     */
    public void setDrawTrackEnabled(boolean ena) {
        drawtrack = ena;
        repaint();
    }
    
    
    /** Main method for standalone testing */
    public static void main(String args[]) {
        JFrame frame = new JFrame();
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                System.exit(0);
            }
        });
        JPanel p = new JPanel();
        p.setBackground(Color.gray);
        JVSlider lvl = new JVSlider();
        lvl.setCommVar("t1");
        lvl.setServer("localhost");
        lvl.setCommtype("org.jvisu.comm.JVSimComm");
        lvl.setMaxVar("1");
        lvl.setMinVar("0");
        lvl.majortics = 5;
        lvl.minortics = 10;
        lvl.setFracDigits(2);
        lvl.minalarm = 20;
        lvl.maxalarm = 80;
        lvl.maxpixels = 20;
        lvl.orientation = 0;
        lvl.setDesignsize(new Dimension(500,500));
        lvl.invers = false;
        lvl.setForeground(Color.BLUE);
        lvl.setExtendbar(0);
        lvl.setLogarithmic(false);
        lvl.setFont(new Font("Default", 0, 11));
        lvl.setReadonly(false);
        lvl.setEditEnabled(true);
        lvl.setOpaque(true);
        lvl.drawBorder = true;
        p.add(lvl);
        frame.getContentPane().add(p);
        frame.pack();
        frame.setVisible(true);
    }    
}
