package org.jvisu.comm;

import java.util.*;
import org.jvisu.comm.var.*;

/** JVisu simulated communication
 *
 *  <p>$Id: JVSimRandomComm.java 168 2003-11-18 08:56:07Z tgl $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *  
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *  <p>This class provides the simulation based communication functions for 
 *  JVisu. This functuionality can be used for prototyping and promotion.
 *  </p>
 *  <p><b>TODO:</b> Check all variable related stuff for thread safety
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0 
 * @since JVisu 1.0
*/
public class JVSimRandomComm extends JVSimComm implements Runnable {
  
    /** Random number generator */    
    private double  random;
    /** the random number generator thread. */    
    private Thread  timer;
    
    /** Creates a new instance of JVCommSim.
     *  Initialize a random generator and start the simulation thread
     */
    public JVSimRandomComm() {
    }
    
    /** Start communication */
    protected void startComm() {
        random = 0;
        active = true;
        timer = new Thread(this);
        timer.start();
        setConnected (true);
        JVComm.setStatusMessage(this, 0, "Started");
    }
    
    /** Stop communication to server.
     * This can only be done, if all variables are unregistered.
     * For simulated communication it means just to stop the
     * simulation thread.
     */    
    protected boolean stopComm() { 
        active = false;
        setConnected (false);
        try { timer.interrupt(); }
        catch (Exception e) {};
        try { timer.join(1000); }
        catch (Exception e) {};
        JVComm.setStatusMessage(this, 0, "Stopped");
        return true;
    }
    
    /** Run the simulation thread.
     * The thread is kept alive as long as the instance is marked active. The
     * method DoSimulation is called every 100ms, which is enough for simulation
     * purposes
     */    
    public void run() {
        while (active) {
            try { 
                Thread.sleep (100); 
                DoSimulation ();
            } catch (InterruptedException e) {}
        }
    }
    
    /** Simulate data by generating random numbers.
     * Walk through the list of registered variables and provide random contents
     * for supported variable types. The values are provided within the given
     * update rate of the variable.
     */
    private void DoSimulation () {
        for (int i = 0; i < vars.size(); i++) {
            JVVar v = (JVVar) vars.get(i);
            v.setElapsed(v.getElapsed() + 100);
            if (v.getElapsed() < v.getVarCfg().getRate())
                continue;
            random += (1.0/37.0);
            if (random > 1.0)
                random = 0;
            v.setElapsed(0);
            switch (v.getVarCfg().getType()) {
                case JVVar.T_DOUBLE:  ((JVVarDouble)v).setVarValue (random); break;
                case JVVar.T_BYTE:    ((JVVarByte)v).setVarValue ((byte) (random * 255)); break;
                case JVVar.T_SHORT:   ((JVVarShort)v).setVarValue ((short) (random * 32767)); break;
                case JVVar.T_INT:     ((JVVarInt)v).setVarValue ((int) (random * 999999)); break;
                case JVVar.T_BOOL:    ((JVVarBool)v).setVarValue (random > 0.5); break;
                default: break;
            }   
        }    
    }
}
