package org.jvisu.comm.var;

import java.nio.*;

/** JVisu variable representing a Boolean.
 *
 * <p>package:  org.jvisu.comm.var</p>
 * <p>file:    JVVarBool.java</p>
 * <p>$Id: JVVarBool.java 275 2004-04-19 08:20:55Z tgl $ </p>
 * <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See file LICENCE in this
 * directory</p>
 *
 * <p>Change Log:</p>
 * <ul>
 * <li> 1.0 Initial inmplimentation </li>
 * </ul>
 * <p>
 * Implementation of abstract class JVVar representing a Boolean variable for
 * communication with a JVisu communication server
 * </p>
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVVarBool extends JVVar {
    
    private boolean value;
    
    /** Creates a new instance of JVVarBool 
     */
    public JVVarBool() {
    }
    
    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public boolean SetVarValue (String str) {
        return setVarValue((Boolean.valueOf(str)).booleanValue());
    }
    
    /** Set the variable value from a bytebuffer
     *  @param buf  bytebuffer containing the variable value (optimized for communication)
     *  @return     true, if successful
    */
    public boolean SetVarValue(ByteBuffer buf) {
        return setVarValue(buf.get() == 1 ? true : false);
    }

    /** Set the variable value from an object
     *  @param obj  Object containing the variable value
     *  @return     true, if successful
    */
    public boolean SetVarValue(Object obj) {
        try {
            boolean val = ((Boolean) obj).booleanValue();
            return setVarValue (val);
        } catch (ClassCastException e) {
            return false;
        }
    }

    /** Set the variable value from a boolean
     * @return true, if successful
     * @param newval the new variable value
     */
    public boolean setVarValue (boolean newval) {
        boolean oldval = value;
        value = newval;
        NotifyListeners(value != oldval);
        return true;
    }
    
    /** Get the variable value as boolean
     *  @return     variable value
     */
    public boolean getVarValue () {
        return value;
    }
    
    /** Get the variable value and store it in a bytebuffer
     *  @param buf  bytebuffer to store variable
     *  @return     modified bytebuffer
     */
    public ByteBuffer GetVarValue(ByteBuffer buf) {
        buf.put((byte)(value ? 1 : 0));
        return buf;
    }
    
    /** Get the string representation of the variable
     *  @return     variable value as string
     */
    public String toString () {
        return Boolean.toString(value);
    }
}
