package org.jvisu.comm.var;

import java.nio.*;

/** JVisu variable representing a IntArray
 *
 * <p>package:  org.jvisu.comm.var</p>
 * <p>file:    JVVarIntArray.java</p>
 * <p>$Id: JVVarIntArray.java 275 2004-04-19 08:20:55Z tgl $ </p>
 * <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See file LICENCE in this
 * directory</p>
 *
 * <p>Change Log:</p>
 * <ul>
 * <li> 1.0 Initial inmplimentation </li>
 * </ul>
 * <p>
 * Implementation of abstract class JVVar representing a byte array variable for
 * communication with a JVisu communication server
 * </p>
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVVarIntArray extends JVVar {
    
    /* Local storage of variable contents */
    int[] value;
    int[] newvalue;
    
    /** Creates a new instance of JVVarFloatArray 
     */
    public JVVarIntArray() {
    }
    
    /** Set the variable configuration 
     *  @param cfg configuration data
     */
    protected void setVarCfg (JVVarCfg cfg) {
        super.setVarCfg(cfg);
        value = new int[cfg.getSize()];
        newvalue = new int[cfg.getSize()];
    }    

    public void setSize(int size)
    {
	if (size == value.length)
		return;

        varcfg.setSize(size);
        value = new int[size];
        newvalue = new int[size];
    }

    
    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public boolean SetVarValue (String str) {
            return false;
    }
    
    /** Set the variable value from a bytebuffer
     *  @param buf  bytebuffer containing the variable value (optimized for communication)
     *  @return     true, if successful
    */
    public boolean SetVarValue(ByteBuffer buf) {
        int len = buf.remaining() / 4;
        if (len > value.length)
            len = value.length;

	int pos = buf.position();

	IntBuffer ibuf = buf.asIntBuffer();
        ibuf.get(newvalue, 0, len);

	buf.position(pos + (len * 4));	

        return setVarValue(newvalue);
    }

    /** Set the variable value from an object
     *  @param obj  Object containing the variable value
     *  @return     true, if successful
    */
    public boolean SetVarValue(Object obj) {
            return false;
    }

    /** Set the variable value from a byte array
     * @return true, if successful
     * @param newval the new variable value
     */
    public boolean setVarValue (int[] newval) {
        int len = value.length < newval.length ? value.length : newval.length;
        boolean changed = false;
        for (int i = 0; i < len; i++) {
            if (value[i] != newval[i])
                changed = true;
            value[i] = newval[i];
        }
        NotifyListeners(changed);
        return true;
    }
    
    /** Get the variable value as byte array
     *  @return     variable value
     */
    public int[] getVarValue () {
        return value;
    }
    
    /** Get the variable value and store it in a bytebuffer
     *  @param buf  bytebuffer to store variable
     *  @return     modified bytebuffer
     */
    public ByteBuffer GetVarValue(ByteBuffer buf) {
	int pos = buf.position();

	IntBuffer ibuf = buf.asIntBuffer();
        ibuf.put(value);

	buf.position(pos + (value.length * 4));
        return buf;
    }
    
    /** Get the string representation of the variable
     *  @return     variable value as string
     */
    public String toString () {
        return "Int array";
    }
}

