package org.jvisu.comm.var;

import java.nio.*;

/** JVisu variable representing a Short
 *
 * <p>package:  org.jvisu.comm.var</p>
 * <p>file:    JVVarShort.java</p>
 * <p>$Id: JVVarShort.java 302 2005-10-16 10:43:06Z ero $ </p>
 * <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See file LICENCE in this
 * directory</p>
 *
 * <p>Change Log:</p>
 * <ul>
 * <li> 1.0 Initial inmplimentation </li>
 * </ul>
 * <p>
 * Implementation of abstract class JVVar representing a short variable for
 * communication with a JVisu communication server
 * </p>
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVVarShort extends JVVar {
    
    short value;
    
    /** Creates a new instance of JVVarShort
     */
    public JVVarShort() {
    }
    
    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public boolean SetVarValue (String str) {
        try {
            return setVarValue(Short.parseShort(str));
        } catch ( NumberFormatException e ) {
            return false;
        }  
    }
    
    /** Set the variable value from a bytebuffer
     *  @param buf  bytebuffer containing the variable value (optimized for communication)
     *  @return     true, if successful
    */
    public boolean SetVarValue(ByteBuffer buf) {
        return setVarValue(buf.getShort());
    }

    /** Set the variable value from an object
     *  @param obj  Object containing the variable value
     *  @return     true, if successful
    */
    public boolean SetVarValue(Object obj) {
        try {
            short val = ((Short) obj).shortValue();
            return setVarValue (val);
        } catch (ClassCastException e) {
            return false;
        }
    }

    /** Set the variable value from a short
     * @return true, if successful
     * @param newval the new variable value
     */
    public boolean setVarValue (short newval) {
        short oldval = value;
        value = newval;
        NotifyListeners((value != oldval));
        return true;
    }
    
    /** Get the variable value as double
     *  @return     variable value
     */
    public short getVarValue () {
        return value;
    }
    
    /** Get the variable value and store it in a bytebuffer
     *  @param buf  bytebuffer to store variable
     *  @return     modified bytebuffer
     */
    public ByteBuffer GetVarValue(ByteBuffer buf) {
        buf.putShort(value);
        return buf;
    }
    
    /** Get the string representation of the variable
     *  @return     variable value as string
     */
    public String toString () {
        return Short.toString(value);
    }
}
