package org.jvisu.comm.var;

import java.nio.*;

/** JVisu variable representing a String.
 *
 * <p>package:  org.jvisu.util.var</p>
 * <p>file:    JVVarString.java</p>
 * <p>$Id: JVVarString.java 270 2004-04-19 06:52:09Z tgl $ </p>
 * <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See file LICENCE in this
 * directory</p>
 *
 * <p>Change Log:</p>
 * <ul>
 * <li> 1.0 Initial inmplimentation </li>
 * </ul>
 * <p>
 * Implementation of abstract class JVVar representing a String variable for
 * communication with a JVisu communication server
 * </p>
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVVarString extends JVVar {
    
    private String value;
    
    /** Creates a new instance of JVVarString
     */
    public JVVarString() {
        value = new String();
    }
    
    /** Set the variable value from a bytebuffer
     *  @param buf  bytebuffer containing the variable value (optimized for communication)
     *  @return     true, if successful
    */
    public ByteBuffer GetVarValue(ByteBuffer buf) {
        buf.put(value.getBytes());
        return buf;
    }
    
    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public boolean SetVarValue(String str) {
        return setVarValue(str);
    }

    /** Set the variable value from a String
     * @return true, if successful
     * @param str String containing the new variable value
     */
    public boolean setVarValue(String str) {
        int isequal = value.compareTo(str);
        value = str;
        NotifyListeners((isequal != 0));
        return true;
    }
    
    /** Get the variable value and store it in a bytebuffer
     *  @param buf  bytebuffer to store variable
     *  @return     modified bytebuffer
     */
    public boolean SetVarValue(ByteBuffer buf) {
        char c;
        String str = "";
        while (buf.hasRemaining()) {
            c = (char) buf.get();
            if (Character.isDefined(c) && !Character.isISOControl(c))
                str += c;
            else
                break;
        }
        return setVarValue (str);
    }

    /** Set the variable value from an object
     *  @param obj  Object containing the variable value
     *  @return     true, if successful
    */
    public boolean SetVarValue(Object obj) {
        try {
            String val = (String) obj;
            return setVarValue (val);
        } catch (ClassCastException e) {
            return false;
        }
    }
    
    /** Get the string representation of the variable
     *  @return     variable value as string
     */
    public String toString () {
        return value;
    }
}
