/*
 * JVPanel.java
 *
 * Created on 15. September 2003, 08:11
 */

package org.jvisu.panel;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.base.*;
import org.jvisu.comm.var.*;

/**
 *  <p>package:    org.jvisu.panel</p>
 *  <p>file:       JVLink.java</p>
 * $Id: JVLinkPanel.java 275 2004-04-19 08:20:55Z tgl $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>g
 * <li> 1.0 First implementation
 * <li> 1.2 Changes due to new variable and communication handling
 * <li> 1.3 Support absolute component placing with respect to the
 *          fact, that array based component creation must be possible.
 * </ul>
 * <p>
 * </p>
 * JVLink is a transparent component, which can be placed anywhere on the 
 * panel. 
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVLinkPanel extends JVSubPanel {

    /** The link */
    private String                  link;
    /** Select, if this is a configfile reload or a html link  */
    private boolean                 htmllink;
    /** Select, if the link is controlled by a variable  */
    private boolean                 linkByVariable;
    
    /** Creates a new instance of JVLabel */
    public JVLinkPanel() {
        super();
        addCommVar (new JVVarCfg(null, updaterate, JVVar.T_BOOL, 0, 2));
        setEnabled(true);
        addPropertyChangeListener(this);
        addMouseListener (new MouseAdapter() {
            /** Mouse is clicked. try to process the link
              * @param e The mouse event
              */
            public void mouseClicked(MouseEvent e) {
                processLink();
            }
        });
    }

    /** This is called, if one of the registered variables
     *  has been changed.
     *  @param var the registered communication variable
     */
    public void VarChanged(org.jvisu.comm.var.JVVar var) {
        super.VarChanged(var);

        if (getCommVarCfg(2).getSymbol() != null &&
            getCommVarCfg(2).getSymbol().equals(var.getVarCfg().getSymbol())) {
            
            if (linkByVariable && ((JVVarBool) var).getVarValue())
                processLink();
            
        }
    }

    /** Process the link
     */
    public void processLink() {
        if (link == null)
            return;
        try {
            Component c = getParent();
            for (; ; c = c.getParent()) {
                if (c instanceof org.jvisu.panel.JVBasePanel && !htmllink) {
                    if (!(c.getClass().getName().equals("org.jvisu.panel.configurator.JVPanelEdit")))
                        ((org.jvisu.panel.JVBasePanel)c).setConfigFile(link);
                }
                if (c instanceof org.jvisu.applets.JVPanel && htmllink) {
                    org.jvisu.applets.JVPanel app = (org.jvisu.applets.JVPanel)c;
                    URL url = new URL(app.getDocumentBase(), link);
                    app.getAppletContext().showDocument(url);
                }
            }
        } catch (Exception e) { }
    }

   /** Control, if link function is controlled by a variable
     *  @param val true, if link function is controlled by a variable
    */
    public void setLinkByVariable (boolean val) {
        linkByVariable = val;
    }

    /** Return setting, if link function is controlled by a variable
     *  @return true, if link function is controlled by a variable
    */
    public boolean getLinkByVariable () {
        return linkByVariable;
    }

    /** Get the symbol of the communication variable, which 
     *  controls the link function
     * @return the variable symbol of this component
     */
    public String getLinkVar() {
        return getCommVarSymbol(2);
    }
    
    /** Initialize commvariable, which controls the link function
     * @param symbol the variable symbol or null
     */
    public void setLinkVar(String symbol){
        setCommVarSymbol(2, symbol);
    }

    /** Get the link
     * @return link
     */
    public String getLink() {
        return link;
    }

    /** Set the link
     * @param link the new link
     */
    public void setLink(String link) {
        this.link = link;
        if (link == null)
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        else
            setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    }

    /** Get the type of link (html or configfile)
     * @return true, if this is a html redirect
     */
    public boolean isHtmlLink() {
        return htmllink;
    }

    /** Set the type of link (html or configfile)
     * @param type true, if this is a html redirect
     */
    public void setHtmlLink(boolean type) {
        htmllink = type;
    }    
}
