package org.jvisu.panel.configurator;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.jvisu.panel.*;
import org.jvisu.util.*;

/** Configurator menu
 *  <p>package:    org.jvisu.panel.configurator</p>
 *  <p>file:       JVPConfMenu.java</p>
 * $Id: JVPConfMenu.java 318 2007-08-21 11:23:19Z mol $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * Class to hold menu descriptor, actions...
 * Reads all the information from a XML file.
 * Language selection support
 * </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVPConfMenu extends JMenuBar {
    
    /** The menufont */
    protected static Font   menufont = new Font("Dialog", Font.PLAIN, 12);
    /** The toolbar, constructed by configfile */
    protected JToolBar      toolBar = new JToolBar();
    /** The popup menu, constructed by configfile */
    protected JPopupMenu    popUp = new JPopupMenu();
    
    /** Create a new instance of JVPanelPaletteMenu
     */
    public JVPConfMenu() {
    }
    
    /* Return the popupmenu, which is configured by the setup
     *  @return popup
     */
    public JPopupMenu getPopup() {
        return popUp;
    }
    
    /* Return the toolbar, which is configured by the setup
     *  @return toolbar
     */
    public JToolBar getToolbar() {
        return toolBar;
    }

    
    /** Subclass for actions related to menus
     */
    public class MAction extends AbstractAction {
        
        /** Storage for text definitions */
        private ArrayList   text;
        
        /** Create a new instance */
        public MAction() {
            JVLanguageSelector.addLanguageChangeListener(this);
            text = new ArrayList();
            text.add("");
        }

        /** Actionlistener is requested by AbstractAction */
        public void actionPerformed(java.awt.event.ActionEvent e) {
            if (e.getActionCommand().compareTo("LanguageSelectionChanged") == 0) {
                this.putValue(Action.NAME, toString());
            }
        }
        
        /** overrides getValue in AbstractAction to provide
         *  multi language support
         */
        public Object getValue(String key) {
            if (key.compareTo(Action.NAME) == 0)
                return toString();
            return super.getValue(key);
        }
        
        /** override toString to provide multilanguage support
         */
        public String toString() {
            try {
                return (String) text.get(JVLanguageSelector.getLanguageSelector());
            } catch (IndexOutOfBoundsException e) {
                return (String) text.get(0);
            }
        }
        
        /** Set the text for a specific language
         *  Copy the default entry, if there is a gap
         *  @param t    text
         *  @param idx language index
         */
        public void setTextForLanguage(String t, int idx) {
            /* Replace an existing entry ? */
            if (idx < text.size()) {
                text.set(idx, t);
            } else {
                for (int i = text.size(); i < idx; i++)
                    text.add(text.get(0));
                text.add(t);
            }
            if (idx == JVLanguageSelector.getLanguageSelector()) {
                this.putValue(Action.NAME, t);
                this.putValue(Action.SHORT_DESCRIPTION, t);
            }
        }
    }
    
    /** Read a menu configuration file
     *  @param file the file, which msut be parsed
     */
    public void parseConfigFile(URL file, ActionListener listener)
    throws Exception {
        
        MenuItemHandler handler = new MenuItemHandler(this, listener);
        URLConnection ctrlConnection = file.openConnection();
        InputStream in = ctrlConnection.getInputStream();
        SAXParser p = (SAXParserFactory.newInstance()).newSAXParser();
        p.parse(in, handler);
        in.close();
    }
    
    /** The xml handler for menu configuration
     */
    private class MenuItemHandler extends org.xml.sax.helpers.DefaultHandler {
        
        /** The current indentation level */
        private int                 level = 0;
        /** Action for the current entry */
        private MAction             action;
        /** current qName */
        private String              qname;
        /** The top menubar */
        private JVPConfMenu         menubar;
        /** the actual item */
        private JComponent          current;
        /** the actual texitem */
        private String              text;
        /** the language for this text */
        private int                 language;
        /** the listener for this action */
        private ActionListener      listener;
        /** popupr items in this category */
        private int                 popupitems = 0;
        /** toolbar items in this category */
        private int                 toolitems = 0;
        
        /** The palette panel, which must be configured */
        
        /** Creates a new instance of ItemHandler */
        public MenuItemHandler(JVPConfMenu bar, ActionListener listener) {
            menubar = bar;
            current = menubar;
            this.listener = listener;
        }
        
        /** Decode the content */
        private String Decode(String content) {
            try {
                return java.net.URLDecoder.decode(content, "UTF-8");
            } catch (Exception e) {
                System.out.println(e);
                System.out.println(content);
            }
            return content;
        }
        
        /** The start of an element
         */
        public void startElement(String uri, String localName, String qName, Attributes attributes) {
            qname = qName;
            level++;
            boolean create = false;
            /* Start a new menu */
            if (qName.compareTo("menu") == 0) {
                if (popupitems > 0)
                    menubar.popUp.addSeparator();
                if (toolitems > 0)
                    menubar.toolBar.addSeparator();
                create = true;
            }
            /* Start a new menuitem */
            if (qName.compareTo("menuitem") == 0) {
                SplashScreen.advanceSplashProgress();
                create = true;
            }
            /* Create the component first, before associating attributes ! */
            if (create) {
                for (int i = 0; i < attributes.getLength(); i++) {
                    String name = Decode(attributes.getQName(i));
                    String value = Decode(attributes.getValue(i));
                    if (name.compareTo("object") == 0) {
                        JComponent c = JVBasePanel.createComponent(value);
                        current.add(c);
                        current = c;
                        c.setFont(menufont);
                        if (current instanceof AbstractButton) {
                            action = new MAction();
                            ((AbstractButton)current).setAction(action);
                            if (listener != null)
                                ((AbstractButton)current).addActionListener(listener);
                        } else
                            action = null;
                        break;
                    }
                }
            }
            for (int i = 0; i < attributes.getLength(); i++) {
                String name = Decode(attributes.getQName(i));
                String value = Decode(attributes.getValue(i));
                if (name.compareTo("command") == 0) {
                    if (action != null) {
                        action.putValue(Action.ACTION_COMMAND_KEY, value);
                        menubar.getActionMap().put(value, action);
                    }
                }
                if (name.compareTo("lan") == 0) {
                    language = org.jvisu.util.param.JVParams.StringToInt(value, -1);
                }
                if (name.compareTo("accel") == 0) {
                    if (action != null) {
                        action.putValue(Action.ACCELERATOR_KEY, javax.swing.KeyStroke.getAWTKeyStroke(value));
                    }
                }
                if (name.equals("enabled") && value.equals("false")) {
                    if (action != null)
                        action.setEnabled(false);
                }
                if (name.equals("popup") && value.equals("true")) {
                    if (action != null) {
                        AbstractButton but;
                        if (current instanceof JCheckBoxMenuItem)
                            but = new JCheckBoxMenuItem(action);
                        else
                            but = new JMenuItem(action);
                        but.setFont(menufont);
                        but.addActionListener(listener);
                        menubar.popUp.add(but);
                        popupitems++;
                    }
                }
                if (name.equals("icon")) {
                    if (action != null) {
                        ImageIcon img;
                        try {
                            img = new ImageIcon(getClass().getResource("/org/jvisu/panel/configurator/icons/" + value));
                            action.putValue(Action.SMALL_ICON, img);
                        } catch (Exception e) { }
                    }
                }
                if (name.equals("toolbar") && value.equals("true")) {
                    if (action != null) {
                        AbstractButton but;
                        if (current instanceof JCheckBoxMenuItem)
                            but = new JToggleButton(action);
                        else
                            but = new JButton(action);
                        but.setFont(new Font("Default", 0, 0));
                        but.setIconTextGap(0);
                        but.setRolloverEnabled(true);
                        but.addActionListener(listener);
                        menubar.toolBar.add(but);
                        toolitems++;
                    }
                }
            }
        }
        
        public void endElement(String uri, String localName, String qName) {
            level--;
            if (qName.compareTo("text") == 0) {
                if (action != null && language >= 0) {
                    action.setTextForLanguage(text, language);
                    ((AbstractButton)current).setAction(action);
                }
            } else {
                if (!(current instanceof JMenuBar)) {
                    if (current instanceof AbstractButton) {
                        ((AbstractButton)current).setAction(null);
                        ((AbstractButton)current).setAction(action);
                    }
                    current = (JComponent) current.getParent();
                    if (current == null)
                        current = menubar;
                }
                if (current instanceof AbstractButton)
                    action = (MAction) ((AbstractButton)current).getAction();
                else
                    action = null;
            }
        }
        
        public void processingInstruction(String target, String data) {
            System.out.println("processing instruction: " + target);
        }
        
        public void characters(char[] ch, int start, int length) {
            if (qname.compareTo("text") == 0)
                text = Decode(new String(ch, start, length));
        }
        
        public void warning(SAXParseException e) {
            System.out.println("Warning:" + (e.getMessage() != null
            ? e.getMessage()
            : e.getClass().getName()));
        }
        
        public void error(SAXParseException e) {
            System.out.println("Error:" + (e.getMessage() != null
            ? e.getMessage()
            : e.getClass().getName()));
        }
    }
    
    /** Main method for standalone testing */
    public static void main(String args[]) {
        JFrame frame = new JFrame();
        JVPConfMenu m = new JVPConfMenu();
        
        try {
            URL url = new URL("file:///home/thomas/work/JVisu/tmp/configmenus.xml");
            m.parseConfigFile(url, null);
        } catch (Exception e) {
            System.exit(-1);
        }
        frame.setJMenuBar(m);
        frame.pack();
        frame.setVisible(true);
    }
}
