/*
 * A password dialog box.
 *
 * Derived from PasswordDialog written by
 * Copyright (C) 2001, 2002 Stephen Ostermiller 
 * http://ostermiller.org/contact.pl?regarding=Java+Utilities
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

package org.jvisu.panel.configurator.dialogs;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A modal dialog to display the about information
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 */
public class JVAboutDlg extends JDialog {

    /** The OK button. */
    protected JButton okButton;

    /** Create this dialog with the given parent
     * @param parent window from which this dialog is launched
     */
    public JVAboutDlg(Frame parent) {
        this(parent, null);
    }

    /** Create this dialog with the given parent and title.
     * @param parent window from which this dialog is launched
     * @param title the title for the dialog box window
     */
    public JVAboutDlg(Frame parent, String title) {
        super(parent, title, true);
        if (title==null)
            setTitle("About");
        if (parent != null)
            setLocationRelativeTo(parent);
    }

    /** Called by constructors to initialize the dialog.
     */
    protected void dialogInit(){

        okButton = new JButton("OK");
        super.dialogInit();

        KeyListener keyListener = (new KeyAdapter() {
            public void keyPressed(KeyEvent e){
                if (e.getKeyCode() == KeyEvent.VK_ESCAPE ||
                    e.getKeyCode() == KeyEvent.VK_ENTER)
                setVisible(false);
            }
        });
        addKeyListener(keyListener);

        ActionListener actionListener = new ActionListener() {
            public void actionPerformed(ActionEvent e){
                Object source = e.getSource();
                setVisible(false);
            }
        };

        JPanel pane = new JPanel(new BorderLayout());
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
        panel.add(new Label("JVisu Panel Configurator"));
        panel.add(new Label("Version 1.0"));
        panel.add(new Label("(C) The JVisu team"));
        panel.add(new Label("Authors: Thomas Gleixner et al"));
        pane.add(panel,BorderLayout.CENTER);

        okButton.addActionListener(actionListener);
        okButton.addKeyListener(keyListener);
        panel = new JPanel();
        panel.add(okButton);
        pane.add(panel,BorderLayout.SOUTH);
        getContentPane().add(pane);

        pack();
    }

    /** Shows the dialog and returns true if the user pressed ok.
     */
    public boolean showDialog() {
        setVisible(true);
        return true;
     }
}

