package org.jvisu.panel.configurator.editors;

import java.awt.*;
import java.beans.*;

/**
 *  <p>package:    JVisu</p>
 *  <p>file:       JVSubPanel.java</p>
 * $Id: JVDimensionEditor.java 153 2003-11-13 19:15:01Z tgl $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 * JVSubPanel is a empty subclass of JVPanel. The only purpose is to
 * identify the size optimzizable components in JVPanel.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVDimensionEditor extends PropertyEditorSupport {
    
    /** The dimension object */
    private Dimension   dim;
    
    /** Creates a new instance of JVDimensionEditor */
    public JVDimensionEditor() {
        dim = new Dimension();
    }

    /** Get the property value as text 
     *  @return string representation
     */
    public String getAsText() {
        return (dim.width + "," + dim.height);
    }

    /** Set the property value as text 
     *  @param value string representation
     */
    public void setAsText(String value) {
        int sep = value.indexOf(',');
        if (sep < 0)
            return;
        try {
            int w = Integer.parseInt(value.substring(0,sep).trim());
            int h = Integer.parseInt(value.substring(sep+1).trim());
            dim.width = w;
            dim.height = h;
        } catch (Exception e) { return; }
    }
    
    /** Set the property value 
     *  @param obj Object, which sets the value
     */
    public void setValue(Object obj) {
        try {
            dim = new Dimension((Dimension) obj);
        } catch (ClassCastException e) { return; }
    }

    /** Get the property value as object
     *  @return  Object, which sets the value
     */
    public Object getValue() {
        return dim;
    }
}
