package org.jvisu.panel.configurator.events;

import java.util.*;

/** Editor events for undo / redo
 *
 *  <p>$Id: JVEditEventSupport.java 212 2003-11-21 16:56:48Z tgl $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *  
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *  <p>Editor event for undo / redo with convenience classes for storing
 *     the event information depending on the event type.
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0 
*/
public class JVEditEventSupport extends Object {

    /* Event listeners */
    ArrayList           listeners = new ArrayList();
    
    /** Create a new instance of JVEditEvent
     */
    public JVEditEventSupport() {
    }

    /** Add an listener 
     *  @param listener Listener to add to the listener list
     */
    public void addEditEventListener(JVEditEventListener listener) {
        if (listener != null)
            listeners.add(listener);
    }

    /** Remove an listener 
     *  @param listener Listener to remove from the listener list
     */
    public void removeEditEventListener(JVEditEventListener listener) {
        if (listener != null)
            listeners.remove(listener);
    }
    
    /** Fire an editor event 
     *  @param evt the new editor event, which should be fired
     */
    public void fireEditEvent (JVEditEvent evt) {
        for (int i = 0; i < listeners.size(); i++)
            ((JVEditEventListener)listeners.get(i)).editEventFired(evt);
    }
}
