/*
 * JVCronSelect.java
 *
 * Created on 18. Oktober 2003, 18:08
 */

package org.jvisu.util;

import java.awt.*;
import javax.swing.*;
import sun.misc.*;

/** Helper class for config editor to edit crons
 *
 *  <p>package:  org.jvisu.util</p>
 *  <p>file:    JVCronSelect.java</p>
 *  <p>$Id: JVCronSelect.java 318 2007-08-21 11:23:19Z mol $ </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *  
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *	Cron job editor
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0 
*/
public class JVCronSelect extends javax.swing.JPanel {
    
    private JCheckBox[]	days;
    private String[]	dayText = {"Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"};
    private JComboBox   interval;
    private String[]    intervalText = {"Nicht aktiv", "Tglich", "Wchentlich", "Monatlich"};
    private JComboBox   starth, startm, repeath, repeatm, repeatnum;
    private JPanel	p1,p2;
    
    /** Creates new form JVCronSelector */
    public JVCronSelect() {
        initComponents();
    }
    
    public void setEnabled (boolean b) {
    	super.setEnabled(b);
	Component[] c = p1.getComponents();
	for (int i = 0; i < c.length; i++)
		c[i].setEnabled(b);
	c = p2.getComponents();
	for (int i = 0; i < c.length; i++)
		c[i].setEnabled(b);
    }	

    public void setSelections (String sel) {
        int i;
        try {
            for (i = 0; i < 7; i++) {
                if (sel.charAt(i) == 'Y')
                    days[i].setSelected(true);
                else
                    days[i].setSelected(false);
            }
            // 7 is a delimiter
            interval.setSelectedIndex(Integer.parseInt(sel.substring(8,9)));
            // 9 is a delimiter
            starth.setSelectedIndex(Integer.parseInt(sel.substring(10,12)));
            // 12 is a delimiter
            startm.setSelectedIndex(Integer.parseInt(sel.substring(13,15)) / 5);
            // 15 is a delimiter
            repeath.setSelectedIndex(Integer.parseInt(sel.substring(16,18)));
            // 18 is a delimiter
            repeatm.setSelectedIndex(Integer.parseInt(sel.substring(19,21)) / 5);
            // 21 is a delimiter
            repeatnum.setSelectedIndex(Integer.parseInt(sel.substring(22,23)));
            
        } catch (Exception e) {}
    }
    
    public String getSelections () {
        String txt = "";
        int sel;

        for (int i = 0; i < 7; i++)
            txt += days[i].isSelected() ? "Y" : "N";

        txt += ";";
        sel = interval.getSelectedIndex();
        txt += Integer.toString(sel);

        txt += ";";
        sel = starth.getSelectedIndex();
        if (sel < 10)
            txt += "0";
        txt += Integer.toString(sel);
        txt += ":";
        sel = startm.getSelectedIndex() * 5;
        if (sel < 10)
            txt += "0";
        txt += Integer.toString(sel);

        txt += ";";
        sel = repeath.getSelectedIndex();
        if (sel < 10)
            txt += "0";
        txt += Integer.toString(sel);
        txt += ":";
        sel = repeatm.getSelectedIndex() * 5;
        if (sel < 10)
            txt += "0";
        txt += Integer.toString(sel);

        txt += ";";
        sel = repeatnum.getSelectedIndex();
        txt += Integer.toString(sel);
        
        return txt;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        int i;
        days = new JCheckBox[7];
        for (i = 0; i < 7; i++)
            days[i] = new JCheckBox(dayText[i]);
        interval = new JComboBox(intervalText);
        starth = new JComboBox();
        startm = new JComboBox();
        repeath = new JComboBox();
        repeatm = new JComboBox();
        repeatnum = new JComboBox();
        for (i = 0; i < 24; i++) {
            starth.addItem(Integer.toString(i));
            repeath.addItem(Integer.toString(i));
        }
        for (i = 0; i < 60; i+=5) {
            startm.addItem(Integer.toString(i));
            repeatm.addItem(Integer.toString(i));
        }
        for (i = 0; i < 10 ; i++)
            repeatnum.addItem(Integer.toString(i));
        
        setLayout(new BoxLayout(this, javax.swing.BoxLayout.X_AXIS));
        p1 = new JPanel();
        p1.setLayout(new BoxLayout(p1, javax.swing.BoxLayout.Y_AXIS));
        for (i = 0; i < 7; i++)
            p1.add(days[i]);
        
        p2 = new JPanel();
        SpringLayout lay = new javax.swing.SpringLayout();
        p2.setLayout(lay);
        JLabel lbl1 = new JLabel("Intervall");
        p2.add (lbl1);
        p2.add(interval);
        JLabel lbl2 = new JLabel("Startzeit");
        p2.add (lbl2);
        p2.add(starth);
        p2.add(startm);
        JLabel lbl3 = new JLabel("Wiederholzeit");
        p2.add (lbl3);
        p2.add(repeath);
        p2.add(repeatm);
        JLabel lbl4 = new JLabel("Wiederholungen");
        p2.add (lbl4);
        p2.add(repeatnum);

        int height = 0;
        int width = 0;
        for (i = 0; i < 8; i++) {
            SpringLayout.Constraints constr = lay.getConstraints(p2.getComponent(i));
            if (constr.getHeight().getValue() > height)
                height = constr.getHeight().getValue();
            if (constr.getWidth().getValue() > width)
                width = constr.getWidth().getValue();
        }    
        width  += 13;
        
        lay.putConstraint(SpringLayout.WEST, lbl1, 10, SpringLayout.WEST, p2);
        lay.putConstraint(SpringLayout.WEST, lbl2, 0, SpringLayout.WEST, lbl1);
        lay.putConstraint(SpringLayout.WEST, lbl3, 0, SpringLayout.WEST, lbl1);
        lay.putConstraint(SpringLayout.WEST, lbl4, 0, SpringLayout.WEST, lbl1);
        
        lay.putConstraint(SpringLayout.NORTH, lbl1, 10 + 1 * height - lay.getConstraints(lbl1).getHeight().getValue(), SpringLayout.NORTH, p2); 
        lay.putConstraint(SpringLayout.NORTH, lbl2, 10 + 2 * height - lay.getConstraints(lbl2).getHeight().getValue(), SpringLayout.NORTH, p2); 
        lay.putConstraint(SpringLayout.NORTH, lbl3, 10 + 3 * height - lay.getConstraints(lbl3).getHeight().getValue(), SpringLayout.NORTH, p2); 
        lay.putConstraint(SpringLayout.NORTH, lbl4, 10 + 4 * height - lay.getConstraints(lbl4).getHeight().getValue(), SpringLayout.NORTH, p2); 

        lay.putConstraint(SpringLayout.WEST, interval , width, SpringLayout.WEST, p2);
        lay.putConstraint(SpringLayout.WEST, starth , width, SpringLayout.WEST, p2);
        lay.putConstraint(SpringLayout.WEST, repeath , width, SpringLayout.WEST, p2);
        lay.putConstraint(SpringLayout.WEST, repeatnum , width, SpringLayout.WEST, p2);
    
        lay.putConstraint(SpringLayout.NORTH, interval, 10 + 1 *height - lay.getConstraints(interval).getHeight().getValue(), SpringLayout.NORTH, p2); 
        lay.putConstraint(SpringLayout.NORTH, starth, 10 +  2 * height - lay.getConstraints(interval).getHeight().getValue(), SpringLayout.NORTH, p2); 
        lay.putConstraint(SpringLayout.NORTH, repeath, 10 + 3 * height - lay.getConstraints(interval).getHeight().getValue(), SpringLayout.NORTH, p2); 
        lay.putConstraint(SpringLayout.NORTH, repeatnum, 10 + 4 * height - lay.getConstraints(interval).getHeight().getValue(), SpringLayout.NORTH, p2); 

        lay.putConstraint(SpringLayout.NORTH, startm , 0, SpringLayout.NORTH, starth);
        lay.putConstraint(SpringLayout.NORTH, repeatm , 0, SpringLayout.NORTH, repeath);
        
        lay.putConstraint(SpringLayout.WEST, startm , 3, SpringLayout.EAST, starth);
        lay.putConstraint(SpringLayout.WEST, repeatm , 3, SpringLayout.EAST, repeath);
 
        p2.setPreferredSize (new java.awt.Dimension(220, 100));
        //p1.setBorder (new javax.swing.border.LineBorder (java.awt.Color.GRAY));
        //p2.setBorder (new javax.swing.border.LineBorder (java.awt.Color.GRAY));
        this.add(p2);
        this.add(p1);
        revalidate();
    }
    
    public static void main(String args[]) {
        JFrame frame = new JFrame();
        frame.getContentPane().add(new JVCronSelect());
        frame.pack();
        frame.setVisible(true);
    }    

}
